@@ -14,8 +14,7 @@
 ## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with this program; if not, write to the Free Software
-## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+## along with this program; if not, see <http://www.gnu.org/licenses/>.
 ##
 
 '''
@@ -23,5 +22,4 @@ This decoder stacks on top of the 'spi' PD and decodes the In-System
 Programming (ISP) protocol of some Atmel AVR 8-bit microcontrollers.
 '''
 
-from .pd import *
-
+from .pd import Decoder
