@@ -14,14 +14,14 @@
 ## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with this program; if not, write to the Free Software
-## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+## along with this program; if not, see <http://www.gnu.org/licenses/>.
 ##
 
 '''
 The SPI (Serial Peripheral Interface) protocol decoder supports synchronous
 SPI(-like) protocols with a clock line, a MISO and MOSI line for data
 transfer in two directions, and an optional CS# pin.
+
 Either MISO or MOSI (but not both) can be optional.
 
 If CS# is supplied, data is only decoded when CS# is asserted (clock
@@ -29,5 +29,4 @@ transitions where CS# is not asserted are ignored). If CS# is not supplied,
 data is decoded on every clock transition (depending on SPI mode).
 '''
 
-from .pd import *
-
+from .pd import Decoder
