@@ -70,7 +70,7 @@ SRD_API int srd_session_new(struct srd_session **sess)
 	/* Keep a list of all sessions, so we can clean up as needed. */
 	sessions = g_slist_append(sessions, *sess);
 
-	srd_dbg("Created session %d.", (*sess)->session_id);
+	srd_dbg("Creating session %d.", (*sess)->session_id);
 
 	return SRD_OK;
 }
@@ -96,9 +96,9 @@ SRD_API int srd_session_start(struct srd_session *sess)
 	if (!sess)
 		return SRD_ERR_ARG;
 
-	srd_dbg("Calling start() on all instances in session %d.", sess->session_id);
+	srd_dbg("Calling start() of all instances in session %d.", sess->session_id);
 
-	/* Run the start() method on all decoders receiving frontend data. */
+	/* Run the start() method of all decoders receiving frontend data. */
 	ret = SRD_OK;
 	for (d = sess->di_list; d; d = d->next) {
 		di = d->data;
@@ -372,7 +372,8 @@ SRD_API int srd_pd_output_callback_add(struct srd_session *sess,
 	if (!sess)
 		return SRD_ERR_ARG;
 
-	srd_dbg("Registering new callback for output type %d.", output_type);
+	srd_dbg("Registering new callback for output type %s.",
+		output_type_name(output_type));
 
 	pd_cb = g_malloc(sizeof(struct srd_pd_callback));
 	pd_cb->output_type = output_type;
