@@ -14,13 +14,15 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef LIBSIGROKDECODE_TESTS_LIB_H
 #define LIBSIGROKDECODE_TESTS_LIB_H
 
+void srdtest_setup(void);
+void srdtest_teardown(void);
+
 Suite *suite_core(void);
 Suite *suite_decoder(void);
 Suite *suite_inst(void);
